# 180515 - plot length distribution

setwd("~/Documents/UNI_und_VORLESUNGEN/14 Guelph/1 Primer Validation/1 Lab Work/2 gradient fusion primers")

library("stringdist")



# make different tagging versions based on script make_degeneracy_v4.R


data <- read.csv("imput_tagging.txt", sep="\t", stringsAsFactors=F)


# Filling primers up semi randomly

temp4 <- NULL

for (i in 1:nrow(data)){
temp4 <- rbind(temp4, unlist(strsplit(data$Tag[i], "")))
}


# nucleotide table
upac <- read.csv(text=c("ID,comment,A,T,C,G, comp
A,Adenine,1,0,0,0,T
C,Cytosine,0,0,1,0,G
G,Guanine,0,0,0,1,C
T,Thymine,0,1,0,0,A
R,A or G,0.5,0,0,0.5,Y
Y,C or T,0,0.5,0.5,0,R
S,G or C,0,0,0.5,0.5,S
W,A or T,0.5,0.5,0,0,W
K,G or T,0,0.5,0,0.5,M
M,A or C,0.5,0,0.5,0,K
B,C or G or T,0,0.3,0.3,0.3,V
D,A or G or T,0.3,0.3,0,0.3,H
H,A or C or T,0.3,0.3,0.3,0,D
V,A or C or G,0.3,0,0.3,0.3,B
N,any base,0.25,0.25,0.25,0.25,N
I,inosine,0.25,0.25,0.25,0.25,N
-,gap,0,0,0,0,-"), stringsAsFactors=F)

upac[upac==0.3] <- 1/3


upac2 <- upac
upac2[3:6] <- upac2[3:6]>0
upac2 <- cbind(upac2, "sum"=rowSums(upac2[3:6]))


tag <- 7

comp <- NULL 

for(i in 1:ncol(temp4)){
meep <- colSums(upac[match(temp4[,i], upac$ID), 3:6], na.rm=T)
comp <- rbind(comp, meep)
}

pdf("Primer_bases_v1.pdf")
x <- barplot(t(comp), col=c("Red", "lawngreen", "Yellow", "dodgerblue"), names=1:tag, main="Primer base composition")

text(x, colSums(t(comp)[1:3,])+t(comp)[4,]/2, round(t(comp)[4,], 1))
text(x, colSums(t(comp)[1:2,])+t(comp)[3,]/2, round(t(comp)[3,], 1))
text(x, t(comp)[1,]+t(comp)[2,]/2, round(t(comp)[2,], 1))
text(x, t(comp)[1,]/2, round(t(comp)[1,], 1))


dev.off()


# run g itterations
EXP <- NULL
mycount <- NULL
for (g in 1:500){
temp2 <- temp4
# fill bases that are numeric with proportionately random numbers

for (i in 1:ncol(temp2)){
replace <- suppressWarnings(!is.na(as.numeric(temp2[,i], warn=F)))

sampleset <- NULL
for (m in 1:4){
abund_nuc <- nrow(temp2)/4-round(comp[i,m])+0.5
if (abund_nuc < 0){abund_nuc<- 0}
sampleset <- c(sampleset, rep(c("A", "T", "C", "G")[m], abund_nuc))
}

temp2[,i][replace] <- sample(sampleset, replace=F)[1:sum(replace)]

}

i <- 2
sum(replace)
length(sampleset)


# plot bases
tempCol <- temp2

tempCol[tempCol =="A"] <- "Red"
tempCol[tempCol =="T"] <- "lawngreen"
tempCol[tempCol =="G"] <- "Yellow"
tempCol[tempCol =="C"] <- "dodgerblue"
tempCol[nchar(tempCol)==1] <- "lightgray"

pdf(paste("1_Primer_plots/", g,"_alignment.pdf", sep=""), height= 9, width=4)

plot(NULL, ylim=c(nrow(temp2), 1), xlim=c(-2,ncol(temp4)+0.5), ylab="", yaxt="n", xaxt="n", xlab="")
axis(1, at=1:nrow(temp2))
text(0.3, 1:nrow(temp2),  data$ID, adj=1)

for (m in 1:ncol(temp2)){

for (j in 1:nrow(temp2)){
if(data$rm[j]>=m){border <- "black"} else {border <- F}
rect(m-0.48, j-0.46, m+0.48, j+0.46, col= tempCol[j,m], border= border, lwd=1.5, )
text(m, j, temp2[j,m])
}

}
dev.off()

#save tags

merge <- NULL
tag_only <- NULL

for (n in 1:nrow(temp2)){
merge[n] <- paste(temp2[n,], collapse="")
tag_only[n] <- paste(temp2[n,1:data$rm[n]], collapse="")
}

exp <- data.frame("ID"=data$ID, "rm"=data$rm, merge, tag_only)

write.csv(exp, file=paste("1_Primer_plots/", g,"_tags_tab.csv", sep=""), row.names=F)

# evaluate missmatches between tags, based on Tag_mm_v9_blue.R


TagLength <- 7

DNA <- exp$merge

DNA <- data.frame("sequ" = unlist(DNA), stringsAsFactors=F)
row.names(DNA) <- exp$ID

#make substring
#count number of missmatches (minimum 2 mm)
DNA$sequ <- substr(DNA$sequ, 1, TagLength)

data2 <- strsplit(DNA$sequ, "")


k <- 1
# save possible primer combination!

save <- NULL

for (k in 1:nrow(DNA)){
ID <- match(data2[[k]], upac2$ID) # k = tag number

deg <- upac2$sum[ID]
sumdeg <- 1
for (i in 1:length(deg)){ # i = base of tag
sumdeg <- sumdeg*deg[i]
}


save_deg <- NULL
for (i in 1:length(data2[[k]])){

keep <- c("A", "T", "C", "G")[unlist(upac2[ID[i], 3:6])]
if (sumdeg>1){
save_deg <- cbind(save_deg, rep(keep, 1, sumdeg))
save_deg <- save_deg[order(save_deg[,i]),]} else{
save_deg[i] <- upac2$ID[ID[i]]
}

}

if(sumdeg>1){
temp <- cbind(row.names(DNA)[k], DNA[k,], save_deg)}else{
temp <- c(row.names(DNA)[k], DNA[k,], save_deg)
}

save <- rbind(save, temp)

}


save

Tags <- save[,-c(1,2)]

collapsed <- NULL
for (s in 1:nrow(Tags)){
collapsed[s] <- paste(Tags[s,], collapse="")
}

cbind(save, collapsed)

table <- stringdistmatrix(collapsed, collapsed, "hamming")


# print likely canidates
if(sum(table==5)==3*4){print(g)}
mycount[g] <- sum(table==5)


# end

col <- rev(c("white","white","white","white", "lightyellow", "Orange", "Red", "Darkred"))

pdf(paste("1_Primer_plots/", g,"_Hamming.pdf", sep=""), width=20, height=20)
par(mar=c(12,12,1,1))

lim <- nrow(table)
plot(-4, xlim=c(2, lim-1), ylim=c(lim-1, 2), xaxt="n", yaxt="n", xlab="", ylab="")

for (i in 1: lim){ # get column
	for (k in 1: lim){ # plot row
		rect(i-0.5, k-0.5, i+0.5, k+0.5, col= col[table[k, i]+1], border=NA)
		text(i, k, table[k, i])
	}
	
}

# primer names including wobble possibilities
name <- NULL
for (i in 1:nrow(save)){
name[i] <- paste(save[i,1], "_", paste(save[i, 3:ncol(save)], sep="", collapse=""), sep="", collapse="")}


axis(1, at=1:lim, name, las=2, family="mono")
axis(2, at=1:lim, name, las=1, family="mono")

# circle primers!

primernames <- as.vector(save[,1])
unique <- unique(primernames)

for (k in 1:length(unique)){
x <- range(which(primernames == unique[k]))
for (i in 1:length(unique)){
y <- range(which(primernames == unique[i]))
rect(x[1]-0.5, y[2]+0.5, x[2]+0.5, y[1]-0.5, lwd=1.5)
}}

dev.off()


# levenshtein lv
table <- stringdistmatrix(collapsed, collapsed, "lv")

pdf(paste("1_Primer_plots/", g,"_Levenshtein.pdf", sep=""), width=20, height=20)
par(mar=c(12,12,1,1))

lim <- nrow(table)
plot(-4, xlim=c(2, lim-1), ylim=c(lim-1, 2), xaxt="n", yaxt="n", xlab="", ylab="")

for (i in 1: lim){ # get column
	for (k in 1: lim){ # plot row
		rect(i-0.5, k-0.5, i+0.5, k+0.5, col= col[table[k, i]+1], border=NA)
		text(i, k, table[k, i])
	}
	
}



EXP <- rbind(EXP, c(g, sum(table<3)))




# primer names including wobble possibilities
name <- NULL
for (i in 1:nrow(save)){
name[i] <- paste(save[i,1], "_", paste(save[i, 3:ncol(save)], sep="", collapse=""), sep="", collapse="")}


axis(1, at=1:lim, name, las=2, family="mono")
axis(2, at=1:lim, name, las=1, family="mono")

# circle primers!

primernames <- as.vector(save[,1])
unique <- unique(primernames)

for (k in 1:length(unique)){
x <- range(which(primernames == unique[k]))
for (i in 1:length(unique)){
y <- range(which(primernames == unique[i]))
rect(x[1]-0.5, y[2]+0.5, x[2]+0.5, y[1]-0.5, lwd=1.5)
}}

dev.off()

print(g)
}




write.csv(EXP, "stats.csv")

EXP <- data.frame(EXP)
head(EXP[order(EXP[,2]),])
plot(EXP[,2])
#plot(density(EXP[,2]))